/**
 * Ext GWT 3.0.0-rc - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client;

import com.sencha.gxt.cell.core.client.ProgressBarCell;
import com.sencha.gxt.widget.core.client.cell.CellComponent;

/**
 * An manual mode updateable progress bar widget.
 * 
 * <p />
 * You are responsible for showing, updating (via {@link #updateProgress}) and
 * clearing the progress bar as needed from your own code. This method is most
 * appropriate when you want to show progress throughout an operation that has
 * predictable points of interest at which you can update the control.
 */
public class ProgressBar extends CellComponent<Double> {

  protected final ProgressBarCell cell;

  /**
   * Creates a new progress bar with the default progress bar cell.
   */
  public ProgressBar() {
    this(new ProgressBarCell());
  }

  /**
   * Creates a new progress bar with the specified progress bar cell.
   * 
   * @param cell the progress bar cell
   */
  public ProgressBar(ProgressBarCell cell) {
    super(cell);
    this.cell = cell;
  }

  /**
   * Returns the bar's increment value.
   * 
   * @return the increment the increment
   */
  public int getIncrement() {
    return cell.getIncrement();
  }

  /**
   * Resets the progress bar value to 0 and text to empty string.
   */
  public void reset() {
    cell.reset(createContext(), getElement());
  }

  /**
   * The number of progress update segments to display within the progress bar
   * (defaults to 10). If the bar reaches the end and is still updating, it will
   * automatically wrap back to the beginning.
   * 
   * @param increment the new increment
   */
  public void setIncrement(int increment) {
    cell.setIncrement(increment);
  }

  /**
   * Updates the progress bar value, and optionally its text. If the text
   * argument is not specified, any existing text value will be unchanged. To
   * blank out existing text, pass "". Note that even if the progress bar value
   * exceeds 1, it will never automatically reset -- you are responsible for
   * determining when the progress is complete and calling {@link #reset} to
   * clear and/or hide the control.
   * 
   * @param value A value between 0 and 1 (e.g., .5, defaults to 0)
   * @param text The string to display in the progress text element or null.
   */
  public void updateProgress(double value, String text) {
    cell.updateProgress(createContext(), getElement(), value, text);
  }

  /**
   * Updates the progress bar text.
   * 
   * @param text the text to display
   */
  public void updateText(String text) {
    cell.updateText(getElement(), text);
  }

  @Override
  protected void onResize(int width, int height) {
    super.onResize(width, height);
    cell.setSize(width, height);
    redraw();
  }

}
