/**
 * Ext GWT 3.0.0-rc - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client.event;

import com.google.gwt.cell.client.Cell.Context;
import com.google.gwt.event.shared.EventHandler;
import com.google.gwt.event.shared.GwtEvent;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.event.shared.HasHandlers;
import com.sencha.gxt.core.shared.event.CancellableEvent;
import com.sencha.gxt.widget.core.client.event.BeforeSelectionEvent.BeforeSelectionHandler;

/**
 * Fires before a selection occurs.
 * 
 * @param <T> the type about to be selected
 */
public class BeforeSelectionEvent<T> extends GwtEvent<BeforeSelectionHandler<T>> implements CancellableEvent {

  /**
   * Handler interface for {@link BeforeSelectionEvent} events.
   * 
   * @param <T> the type about to be selected
   */
  public interface BeforeSelectionHandler<T> extends EventHandler {

    /**
     * Called when {@link BeforeSelectionEvent} is fired. The action can be
     * cancelled using {@link BeforeSelectionEvent#setCancelled(boolean)}.
     * 
     * @param event the {@link BeforeSelectionEvent} that was fired
     */
    void onBeforeSelection(BeforeSelectionEvent<T> event);
  }

  /**
   * A widget that implements this interface is a public source of
   * {@link BeforeSelectionEvent} events.
   * 
   * @param <T> the type about to be selected
   */
  public interface HasBeforeSelectionHandlers<T> extends HasHandlers {
    /**
     * Adds a {@link BeforeSelectionEvent} handler.
     * 
     * @param handler the handler
     * @return the registration for the event
     */
    HandlerRegistration addBeforeSelectionHandler(BeforeSelectionHandler<T> handler);
  }

  /**
   * Handler type.
   */
  private static Type<BeforeSelectionHandler<?>> TYPE;

  /**
   * Gets the type associated with this event.
   * 
   * @return returns the handler type
   */
  public static Type<BeforeSelectionHandler<?>> getType() {
    if (TYPE == null) {
      TYPE = new Type<BeforeSelectionHandler<?>>();
    }
    return TYPE;
  }

  private T item;
  private boolean canceled;
  private Context context;

  /**
   * Creates a new before selection event.
   * 
   * @param context the cell context
   * @param item the item to be selected
   */
  public BeforeSelectionEvent(Context context, T item) {
    this.context = context;
    this.item = item;
  }
  
  /**
   * Creates a new before selection event.
   * 
   * @param item the item to be selected
   */
  public BeforeSelectionEvent(T item) {
    this.item = item;
  }

  @SuppressWarnings({"unchecked", "rawtypes"})
  @Override
  public final Type<BeforeSelectionHandler<T>> getAssociatedType() {
    return (Type) TYPE;
  }

  /**
   * Returns the cell context if event fired from cell
   * 
   * @return the cell context if event fired via cell or null
   */
  public Context getContext() {
    return context;
  }
  
  /**
   * Gets the item.
   * 
   * @return the item
   */
  public T getItem() {
    return item;
  }

  @Override
  public boolean isCancelled() {
    return canceled;
  }

  @Override
  public void setCancelled(boolean cancel) {
    this.canceled = cancel;
  }

  @Override
  protected void dispatch(BeforeSelectionHandler<T> handler) {
    handler.onBeforeSelection(this);
  }

}
