/**
 * Ext GWT 3.0.0-rc - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client.event;

import com.google.gwt.cell.client.Cell.Context;
import com.google.gwt.event.shared.EventHandler;
import com.google.gwt.event.shared.GwtEvent;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.event.shared.HasHandlers;
import com.sencha.gxt.widget.core.client.event.SelectionEvent.SelectionHandler;

/**
 * Fires after a selection occurs.
 * 
 * @param <T> the type about to be selected
 */
public class SelectionEvent<T> extends GwtEvent<SelectionHandler<T>> {

  /**
   * A widget that implements this interface is a public source of
   * {@link SelectionEvent} events.
   * 
   * @param <T> the type about to be selected
   */
  public interface HasSelectionHandlers<T> extends HasHandlers {
    /**
     * Adds a {@link SelectionEvent} handler.
     * 
     * @param handler the handler
     * @return the registration for the event
     */
    HandlerRegistration addSelectionHandler(SelectionHandler<T> handler);
  }

  /**
   * Handler interface for {@link SelectionEvent} events.
   * 
   * @param <T> the type about to be selected
   */
  public interface SelectionHandler<T> extends EventHandler {

    /**
     * Called when {@link SelectionEvent} is fired.
     * 
     * @param event the {@link SelectionEvent} that was fired
     */
    void onSelection(SelectionEvent<T> event);
  }

  /**
   * Handler type.
   */
  private static Type<SelectionHandler<?>> TYPE;

  /**
   * Gets the type associated with this event.
   * 
   * @return returns the handler type
   */
  public static Type<SelectionHandler<?>> getType() {
    if (TYPE == null) {
      TYPE = new Type<SelectionHandler<?>>();
    }
    return TYPE;
  }

  private T item;
  private Context context;

  /**
   * Creates a new selection event.
   * 
   * @param context the cell context
   * @param item the selected item
   */
  public SelectionEvent(Context context, T item) {
    this.context = context;
    this.item = item;
  }

  /**
   * Creates a new selection event.
   * 
   * @param the selected item
   */
  public SelectionEvent(T item) {
    this.item = item;
  }

  @SuppressWarnings({"unchecked", "rawtypes"})
  @Override
  public final Type<SelectionHandler<T>> getAssociatedType() {
    return (Type) TYPE;
  }

  /**
   * Returns the cell context if event fired from cell
   * 
   * @return the cell context if event fired via cell or null
   */
  public Context getContext() {
    return context;
  }

  /**
   * Gets the item.
   * 
   * @return the item
   */
  public T getItem() {
    return item;
  }

  @Override
  protected void dispatch(SelectionHandler<T> handler) {
    handler.onSelection(this);
  }

}
